function makeMovie(name, str, simtime, movietime, fview, hd)
% makeMovie: Creates an animation and stores it in an .avi movie file.
%
% Parameters:
% name           filename
% str            list of initial l-system letters; or precomputed slides
% simtime        simulation time (in days)
% movietime      length of the movie (in seconds)
% (fview)        callback function after the figure is drawn (default:
%                @(frame) view(0,0)), e.g. for rotation 
% (hd)           hd=0, 1080 lines, 50 fps, hd=1, 720 lines, 24 fps
%               (default = 0).
%
% See also: plotTubes, completeParams, applyRules 
%
% Copyright 2012-2014 Daniel Leitner & Andrea Schnepf. See license.txt for details.
%

global geometry;

if nargin<5
    fview = @(frame) view(0,0);
end

if nargin<6
    hd=0;
end
    
if iscell(str) % slides
    slides = str; % rename
    nof = length(slides);
    FPS = max(12,round(nof/movietime)); % approximate fps
    
else % precompute slides
    if hd 
        FPS = 50; 
    else 
        FPS = 30;
    end
    nof = movietime*FPS; % number of frames   
    dt = simtime/(nof-1);
    slides{1}=str;
    for i = 2 : nof
        fprintf('preparing frame %g / %g \n',i,nof);
        slides{i} = applyRules(slides{i-1},dt);
    end
end

% get final bounds and increase 10 percent
b = getBounds(slides{end}); 
inc = 0.01;
b(1) = b(1) - inc*(b(2)-b(1)); % x
b(2) = b(2) + inc*(b(2)-b(1));
b(3) = b(3) - inc*(b(4)-b(3)); % y
b(4) = b(4) + inc*(b(4)-b(3));
b(5) = b(5) - inc*(b(6)-b(5)); % z 
b(6) = b(6) + inc*(b(6)-b(5));

if hd 
    yy = 1080;
else
    yy = 720;
end
xx = 1080; %round((yy/(b(6)-b(5)))*max(b(2)-b(1),b(3)-b(2)));

% precompute geometry boundaries
X = linspace(b(1),b(2),100);
Y = linspace(b(3),b(4),100);
Z = linspace(b(5),b(6),100);
[geom, n] = plotDistFunc(geometry,X,Y,Z);

% set video codec and options
writerObj = VideoWriter(name,'MPEG-4');
set(writerObj,'Quality',90);
set(writerObj,'FrameRate',FPS);
open(writerObj);

for i = 1 : nof

    fprintf('saving frame %g / %g \n',i,nof);
    fig = figure();
    clf;
    title(['Time: ' num2str((i/nof)*simtime,'%2.1f') ' days'],'fontsize',20);
    daspect([1 1 1])
    set(fig,'Position',[10 60 xx yy]) 
    set(fig,'DoubleBuffer','on');    
    
    % plot roots
    plotTubes(slides{i});         
    hold on;    
    set(gca,'color','white')
    set(gcf,'color','white')  
    set(gca,'FontSize',20)
    view(20,20);
    
    % plot bounds
    p=patch(geom);
    set(p,'VertexNormals',n);
    set(p,'FaceColor',[0,0,1],'EdgeColor','none');
    alpha(p,0.2);
        
    % set axis
    axis(b);
    %axis vis3d;
    fview(i/FPS); % callback function for figure view
   
    % capture frame
    pause(1);        
    frame = getframe(fig);
    %export_fig 'test' -'png' -transparent  %-r250% -transparent % -r400 
    %export_fig('-png','-nocrop','-opengl','-r100','mycim')    
    im = imread('mycim.png');
    frame = im2frame(im);
    writeVideo(writerObj,frame);
    close(fig);

end

close(writerObj);

